#ifndef _CAMERA_H_
#define _CAMERA_H_

#include "Matrix.h"
#include "Point.h"
#include "Vector.h"

class Camera
{
protected:
  Vector fCP,
         fVPN,
         fCOA,
         fVUP;

  double fNear;                 //Near clipping plane
  double fFar;                  //Far clipping plane
  double fHeight;               //Height of projection plane

  Matrix fCameraMatrix;
  Matrix fPerspectiveMatrix;
  Matrix fWindowMatrix;

public:
  Camera() : fNear(0.0), fFar(0.0), fHeight(0.0){}
  Camera (double *coa, double *pos, double *vup, double _height_, double _near_, double _far_)
  : fCOA(coa[0], coa[1], coa[2]),
    fCP(pos[0], pos[1], pos[2]),
    fVUP(vup[0], vup[1], vup[2]),
    fVPN(coa[0] - pos[0], coa[1] - pos[1], coa[2] - pos[2]),
    fNear(_near_),
    fFar(_far_),
    fHeight(_height_)
  {
    fVPN = Normalize(fVPN);

    //WindowXform depends on CameraXform, so call CameraXform first
    fCameraMatrix = ViewMatrix(fCOA, fCP, fVUP);
    fPerspectiveMatrix = PerspectiveMatrix(fNear, fFar, fHeight);
    PrepareWindowMatrix ();
  }
  Camera(const Camera& c) : fCP(c.fCP), fVPN(c.fVPN), fCOA(c.fCOA), fVUP(c.fVUP), fNear(c.fNear), fFar(c.fFar), fHeight(c.fHeight), fCameraMatrix(c.fCameraMatrix), fPerspectiveMatrix(c.fPerspectiveMatrix), fWindowMatrix(c.fWindowMatrix){}

  void Set(double *coa, double *pos, double *vup, double _height_, double _near_, double _far_);
  void PrepareWindowMatrix();
  Point ApplyCamera (const Point& in) const;
  Point ApplyPerspective (const Point& in) const;
  Point ApplyWindow (const Point& in) const;

  Camera& operator=(const Camera& c);

  //Meber access functions
  Vector GetCP() const;
  Vector GetVPN() const;
  Vector GetCOA() const;
  Vector GetVUP() const;
  double GetNear() const;
  double GetFar() const;
  double GetHeight() const;

  friend Point ApplyCamera(const Camera& C, const Point& in);
  friend Point ApplyPerspective(const Camera& C, const Point& in);
  friend Point ApplyWindow(const Camera& C, const Point& in);
  friend ostream& operator<<(ostream& s, const Camera& a);
};

#endif
