#ifndef _DEPTHBUFFER_INL_
#define _DEPTHBUFFER_INL_

#include "DepthBuffer.h"

inline DepthBuffer& DepthBuffer::operator=(const DepthBuffer& buf)
{
  fWidth = buf.fWidth;
  fHeight = buf.fHeight;
  SetSize(buf.fWidth, buf.fHeight);
  memcpy(fBuffer, buf.fBuffer, (buf.fWidth * buf.fHeight * sizeof(int)));
}

inline void DepthBuffer::SetSize(int w, int h)
{
  int i;

  if (fBuffer)
  {
    for (i = 0; i < fWidth; i ++)
      delete [] fBuffer[i];
    delete [] fBuffer;
  }

  fWidth = w;
  fHeight = h;

  fBuffer = new double*[fWidth];
  for (i = 0; i < fWidth; i++)
    fBuffer[i] = new double[fHeight];
}

inline void DepthBuffer::Clear(double _val_)
{
  int i, j;

  for (i = 0; i < fWidth; i++)
    for (j = 0; j < fHeight; j++)
      fBuffer[i][j] = _val_;
}


inline bool DepthBuffer::DepthTest(int x, int y, double z, bool set)
{
  bool result;

  if ( x < 0 || x >= fWidth || y < 0 || y >= fHeight)
    return false;

  result = fBuffer[x][y] > z;
  
  if (result && set)
    SetDepth(x, y, z);

  return result;
}

inline void DepthBuffer::SetDepth(int x, int y, double z)
{
  fBuffer[x][y] = z;
}

inline double DepthBuffer::GetDepth(int x, int y) const
{
  return fBuffer[x][y];
}

#endif