#ifndef _ENVIRONMENT_H_
#define _ENVIRONMENT_H_

#include <stdlib.h>

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "Camera.h"
#include "RGBColor.h"
#include "DepthBuffer.h"

class Poly;
class Point;
class Model;
class AmbientLight;
class PointLight;

class Environment
{
public:

  static int WIDTH;
  static int HEIGHT;

  static bool LIGHTSCENE;
  static unsigned RENDERTYPE;
  static const unsigned WIREFRAME;
  static const unsigned FLAT;
  static const unsigned GOURARD;
  static const unsigned PHONG;

  static unsigned SHADOWTYPE;
  static const unsigned NOSHADOW;
  static const unsigned FAKESHADOW;
  static const unsigned ZBUFFERSHADOW;

  static DepthBuffer fDepthBuffer;
  static Camera fCamera;               //Transformations for 3D viewing

  static AmbientLight fAmbientLight;
  static int fNumPointLights;
  static PointLight *fPointLights;

  //Stuff for shadow zbuffer
  static int fCurrentShadowBuffer;
  static int fNumShadowBuffers;
  static DepthBuffer *fShadowBuffers;

  RGBColor fBackgroundColor;

  //Models
  int fNumModels;               //Number of models
  Model *fModels;               //Array of Vertices

  Environment() : fNumModels(0), fModels(NULL), fBackgroundColor(1.0, 1.0, 1.0) {}
  ~Environment();

  //Read the data files
  void ReadEnvironmentFile(const char *file);
  void ReadObjectFiles(const char *file);
  void ReadCameraFile(const char *file);
  void ReadAmbientLightFile(const char *file);
  void ReadPointLightFile(const char *file);
  void ReadShadowFile(const char *file);

  void Render(wxDC& dc);
  void SetBackgroundColor(double r, double g, double b);
  void SetBackgroundColor(const RGBColor& rgb);
};

#endif
