#ifndef _ENVIRONMENT_INL_
#define _ENVIRONMENT_INL_

#include "wx/wxprec.h"
#include "Environment.h"
#include "Model.h"
#include "Poly.h"
#include "Point.h"
#include "PointLight.h"

#include "Model.inl"
#include "Poly.inl"
#include "Point.inl"
#include "PointLight.inl"

#include "Camera.inl"
#include "RGBColor.inl"
#include "DepthBuffer.inl"

inline void Environment::Render(wxDC& dc)
{
  int i;
  Matrix tran1 = TranslationMatrix( -5, -5, 2 );
  Matrix tran2 = TranslationMatrix( 1, 1, 5 );

  //Clear background
  wxColour background(255 * fBackgroundColor[0], 255 * fBackgroundColor[1], 255 * fBackgroundColor[2]);
  wxPen backgroundpen(background, 1, wxSOLID);
  wxBrush backgroundbrush(background, wxSOLID);
  dc.SetPen(backgroundpen);
  dc.SetBrush(backgroundbrush);
  dc.DrawRectangle(0, 0, WIDTH, HEIGHT);

  //Clear depth buffer
  fDepthBuffer.Clear();

  //Default black background
  dc.SetPen(*wxBLACK_PEN);
  dc.SetBrush(*wxTRANSPARENT_BRUSH);

  //Build shadow buffers
  if (SHADOWTYPE == ZBUFFERSHADOW && RENDERTYPE != WIREFRAME)
  {
    int j;
    unsigned store1;
    bool store2;
    Camera C;
    double coa[3], pos[3], vup[3];
    Vector ldir, lpos, cvup;

    for (i = 0; i < fNumShadowBuffers; i++)
    {
      fShadowBuffers[i].Clear();
      fCurrentShadowBuffer = i;

      lpos = fPointLights[i].GetPosition();
      cvup = fCamera.GetVUP();

      for (j = 0; j < 3; j++)
      {
        pos[j] = lpos[j];
        vup[j] = cvup[j];
      }
/*
      if (fPointLights[i].GetLightType() == PointLight::DIRECTIONAL)
      {
        //Using negative Direction as VPN
        ldir = fPointLights[i].GetDirection();
        coa[0] = ldir[0] + lpos[0];
        coa[1] = ldir[1] + lpos[1];
        coa[2] = ldir[2] + lpos[2];

        C.Set(coa, pos, vup, fCamera.GetHeight(), fCamera.GetNear(), fCamera.GetFar());
      }
      else
      {
*/
      ldir = fCamera.GetCOA();
      coa[0] = ldir[0];
      coa[1] = ldir[1];
      coa[2] = ldir[2];
        
      C.Set(coa, pos, vup, fCamera.GetHeight(), fCamera.GetNear(), fCamera.GetFar());
//      }

      store1 = Environment::RENDERTYPE;
      store2 = Environment::LIGHTSCENE;
      RENDERTYPE = Environment::FLAT;
      LIGHTSCENE = false;

      for (j = 0; j < fNumModels; j++)
        fModels[j].Render(C, dc);

      LIGHTSCENE = store2;
      RENDERTYPE = store1;
    }

    //Reset buffer calue
    fCurrentShadowBuffer = -1;
  }

  for (i = 0; i < fNumModels; i++)
  {
    //Assume floor is last, don't draw it's shadow
    unsigned store = SHADOWTYPE;
    if (i == (fNumModels - 1) && SHADOWTYPE == FAKESHADOW)
    {
      SHADOWTYPE = NOSHADOW;
    }

    fModels[i].Render(fCamera, dc);

    SHADOWTYPE = store;
  }
}

inline void Environment::SetBackgroundColor(double r, double g, double b)
{
  fBackgroundColor[0] = r;
  fBackgroundColor[1] = g;
  fBackgroundColor[2] = b;
}

inline void Environment::SetBackgroundColor(const RGBColor& rgb)
{
  fBackgroundColor = rgb;
}

#endif