#ifndef _FAKE_SHADOW_INL_
#define _FAKE_SHADOW_INL_

#include "FakeShadow.h"
#include "Matrix.inl"
#include "Point.inl"

inline double *FakeShadow::GetShadowPlane(double *plane)
{
  memcpy(plane, fPlane, sizeof(fPlane));
  return plane;
}

inline void FakeShadow::FindShadowPlane (const Point& p0, const Point& p1, const Point& p2)
{
  Vector  vec0,
          vec1;

  vec0[0] = p1[0] - p0[0];
  vec0[1] = p1[1] - p0[1];
  vec0[2] = p1[2] - p0[2];

  vec1[0] = p2[0] - p0[0];
  vec1[1] = p2[1] - p0[1];
  vec1[2] = p2[2] - p0[2];

  fPlane[0] = vec0[1] * vec1[2] - vec0[2] * vec1[1];
  fPlane[1] = -(vec0[0] * vec1[2] - vec0[2] * vec1[0]);
  fPlane[2] = vec0[0] * vec1[1] - vec0[1] * vec1[0];
  fPlane[3] = -(fPlane[0] * p0[0] + fPlane[1] * p0[1] + fPlane[2] * p0[2]);
}

inline void FakeShadow::ComputeShadowMatrix (const Vector& lightPos)
{
  fShadowMatrix = ShadowMatrix(fPlane, lightPos);
};

inline Point FakeShadow::ApplyShadow (const Point& p) const
{
  return fShadowMatrix * p;
}

inline Point ApplyShadow(const FakeShadow& s, const Point& p)
{
  return s.ApplyShadow(p);
}

#endif