#ifndef _MODEL_H_
#define _MODEL_H_

#include <stdlib.h>

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "Camera.h"
#include "Matrix.h"
#include "TextureMap.h"

class Vertex;
class Poly;

class Model
{
protected:

  bool fComputeNormals;
  int fNumPolys;
  int fNumVerts;
  Vertex *fVertices;
  Poly *fPolys;
  TextureMap *fTextureMap;

public:

  Model() : fNumPolys(0), fNumVerts(0), fVertices(NULL), fPolys(NULL), fTextureMap(NULL), fComputeNormals(true) {}
  Model(const Model& m) : fNumPolys(m.fNumPolys), fNumVerts(m.fNumVerts), fVertices(m.fVertices), fPolys(m.fPolys), fTextureMap(m.fTextureMap), fComputeNormals(m.fComputeNormals) {}
  ~Model();

  void Render(const Camera& C, wxDC& dc);
  void MakeShadow(const Vector& lightPos);
  void ApplyMatrix(const Matrix& M);
  void CameraTransform(const Camera& C);
  void PerspectiveTransform(const Camera& C);
  void WindowTransform(const Camera& C);
  void Cull(const Vector& V);
  void Cull();
  void ComputeVertexNormals (const char *file = NULL);
  void ComputeVertexColors ();

  void SetComputeVertexNormals(bool yn);
  bool ShouldComputeVertexNormals() const;

  friend Model MakeShadow(const Model& s, const Vector& lightPos);
  friend Model ApplyMatrix(const Model& s, const Matrix& M);
  friend Model CameraTransform(const Model& s, const Camera& C);
  friend Model PerspectiveTransform(const Model& s, const Camera& C);
  friend Model WindowTransform(const Model& s, const Camera& C);
  friend Model Cull(const Model& s, const Vector& V);
  friend Model Cull(const Model& s);
  friend void Render(Model& s, const Camera& C, wxDC& dc);

  friend ostream& operator<<(ostream& s, const Model& a);
  void ReadObjectFile(const char *file);
  void ReadColorFile(const char *file);
  void ReadSurfaceInfoFile(const char* file);
  void ReadTextureFile(const char* file);
  void ReadNormalFile(const char* file);
  void ReadShadowFile(const char* file);
};

#endif
