#ifndef _POINT_H_
#define _POINT_H_

#include <math.h>

class ostream;
class Matrix;

class Point
{
protected:

  double fX;
  double fY;
  double fZ;
  double fH;

public:

  Point() : fX(0.0), fY(0.0), fZ(0.0), fH(0.0) {}
  Point(double _x_, double _y_, double _z_, double _h_ = 1.0) : fX(_x_), fY(_y_), fZ(_z_), fH(_h_) {}
  Point(double _x_) : fX(_x_), fY(_x_), fZ(_x_), fH(_x_) {}
  Point(const Point& v) : fX(v.fX), fY(v.fY), fZ(v.fZ), fH(v.fH) {}

  double& operator[](int _i_)
  {
    if (_i_ == 0)
      return fX;
    else if (_i_ == 1)
      return fY;
    else if (_i_ == 2)
      return fZ;
    else
      return fH;
  }

  const double& operator[](int _i_) const
  {
    if (_i_ == 0)
      return fX;
    else if (_i_ == 1)
      return fY;
    else if (_i_ == 2)
      return fZ;
    else
      return fH;
  }

  void Set(double x, double y, double z, double h = 1.0);
  void Set(const Point& p);

  double GetX() const;
  double GetY() const;
  double GetZ() const;
  double GetH() const;

  void Homogenize();
  friend Point Homogenize(const Point& p);

  Point& operator=(const Point& p);

  friend int operator==(const Point& a, const Point& b);
  friend int operator<(const Point& a, const Point& b);
  friend ostream& operator<<(ostream& s, const Point& a);

  //In the Matrix inline
  friend Point operator*(const Matrix& M, const Point& p);
};

#endif
