#ifndef _POINTLIGHT_H_
#define _POINTLIGHT_H_

#include <memory.h>
#include "RGBColor.h"
#include "Vector.h"
#include "Point.h"

#ifndef DEG_TO_RAD
#define DEG_TO_RAD	0.017453292519943295
#endif

#ifndef RAD_TO_DEG
#define RAD_TO_DEG	57.295779513082323
#endif

class PointLight
{
protected:
  RGBColor fIntensity;
  Vector fPosition;
  Vector fDirection;
  unsigned fLightType;
  double fP;                  //Direction exponent
  double fC1, fC2, fC3;       //Attenuation factors
  unsigned fAttenuationType;

  //Flaps
  double fFlaps[6];     //{{xmin, xmax}, {ymin, ymax}, {zmin, zmax}}
  bool fFlapEnabled[6];
  double fSpotAngle;
  bool fSpotEnabled;

public:

  static const unsigned FLAPMINX;
  static const unsigned FLAPMAXX;
  static const unsigned FLAPMINY;
  static const unsigned FLAPMAXY;
  static const unsigned FLAPMINZ;
  static const unsigned FLAPMAXZ;

  static const unsigned NO_ATTENUATION;
  static const unsigned LINEAR_ATTENUATION;
  static const unsigned QUADRATIC_ATTENUATION;

  static const unsigned DIRECTIONAL;
  static const unsigned NONDIRECTIONAL;

  PointLight()
  : fIntensity(0.0),
  fPosition(0.0),
  fDirection(0.0),
  fP(0.0), fC1(1.0), fC2(0.0), fC3(0.0),
  fAttenuationType(QUADRATIC_ATTENUATION),
  fSpotAngle(120),
  fSpotEnabled(false),
  fLightType(NONDIRECTIONAL)
  {
    memset(fFlaps, 0, sizeof(fFlaps));
    memset(fFlapEnabled, 0, sizeof(fFlapEnabled));
  }
  PointLight(double r, double g, double b, double x, double y, double z, double dx, double dy, double dz) : fIntensity(r, g, b), fPosition(x, y, z), fDirection(dx, dy, dz), fP(0.0), fC1(1.0), fC2(0.0), fC3(0.0), fAttenuationType(QUADRATIC_ATTENUATION), fSpotAngle(120), fSpotEnabled(false), fLightType(NONDIRECTIONAL) {memset(fFlaps, 0, sizeof(fFlaps)); memset(fFlapEnabled, 0, sizeof(fFlapEnabled));}
  PointLight(double c, double v, double d) : fIntensity(c), fPosition(v), fDirection(d), fP(0.0), fC1(1.0), fC2(0.0), fC3(0.0), fAttenuationType(QUADRATIC_ATTENUATION), fSpotAngle(120), fSpotEnabled(false), fLightType(NONDIRECTIONAL) {memset(fFlaps, 0, sizeof(fFlaps)); memset(fFlapEnabled, 0, sizeof(fFlapEnabled));}
  PointLight(const RGBColor& c, const Vector& v, const Vector& d) : fIntensity(c), fPosition(v), fDirection(d), fP(0.0), fC1(1.0), fC2(0.0), fC3(0.0), fAttenuationType(QUADRATIC_ATTENUATION), fSpotAngle(120), fSpotEnabled(false), fLightType(NONDIRECTIONAL) {memset(fFlaps, 0, sizeof(fFlaps)); memset(fFlapEnabled, 0, sizeof(fFlapEnabled));}
  PointLight(const PointLight& p) : fIntensity(p.fIntensity), fPosition(p.fPosition), fDirection(p.fDirection), fP(p.fP), fC1(p.fC1), fC2(p.fC2), fC3(p.fC3), fAttenuationType(p.fAttenuationType), fSpotAngle(p.fSpotAngle), fSpotEnabled(p.fSpotEnabled), fLightType(p.fLightType) {memcpy(fFlaps, p.fFlaps, sizeof(fFlaps)); memcpy(fFlapEnabled, p.fFlapEnabled, sizeof(fFlapEnabled));}

  void SetPosition(double x, double y, double z);
  void SetPosition(const Vector& v);
  void SetIntensity(double r, double g, double b);
  void SetIntensity(const RGBColor& v);
  void SetDirection(double x, double y, double z);
  void SetDirection(const Vector& v);
  void SetDirectionExponent(double p);
  void SetSpotAngle(double angle);
  void SetFlapValue(unsigned flap, double restriction);
  void SetAttenuationValues(double c1, double c2, double c3);
  void SetLightType(unsigned type);

  Vector GetPosition() const;
  RGBColor GetIntensity(const Vector &dir) const;
  Vector GetDirection () const;
  double GetDirectionExponent() const;
  double GetSpotAngle() const;
  double GetFlapValue(unsigned flap) const;
  void GetAttenuationValues(double *c1, double *c2, double *c3);
  unsigned GetLightType() const;

  void EnableSpotLight();
  void DisableSpotLight();
  bool IsSpotLightEnabled() const;

  void EnableFlap(unsigned flap);
  void DisableFlap(unsigned flap);
  bool AreFlapsEnabled() const;
  bool IsFlapEnabled(unsigned flap) const;

  double GetAttenuation(const Point &pos) const;
  bool IsInFlaps(const Point &pos) const;
  bool IsInSpot(const Vector &Dir) const;

  PointLight& operator=(const PointLight& p);
};

#endif