#ifndef _POLY_H_
#define _POLY_H_

#include <stdlib.h>

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "RGBColor.h"
#include "SurfaceInfo.h"
#include "Vertex.h"
#include "FakeShadow.h"

class Camera;
class Matrix;
class TextureMap;
class ostream;

class Poly
{
protected:

  Vertex *fVertices;      //Vertices for 3d polygon
  int fNVerts;             //Number of vertices
  bool fCulled;
  RGBColor fColor;
  SurfaceInfo fSurfaceInfo;
  TextureMap *fTextureMap;
  FakeShadow fFakeShadow;

public:

  //Default constructor needed for subclasses
  Poly () : fCulled(false), fVertices(NULL), fNVerts(0), fColor(0.0, 0.0, 0.0), fTextureMap(NULL) {}
  Poly (double *x, double *y, double *z, int nverts) : fCulled(false), fColor(0.0, 0.0, 0.0), fTextureMap(NULL)
  {
    fVertices = new Vertex[nverts];
    fNVerts = nverts;
    
    for (int i = 0; i < nverts; i++)
    {
      fVertices[i].Set(x[i], y[i], z[i]);
    }
  }
  Poly (const Vertex *pts, int nverts) : fCulled(false), fColor(0.0, 0.0, 0.0), fTextureMap(NULL)
  {
    fVertices = new Vertex[nverts];
    fNVerts = nverts;
    
    for (int i = 0; i < nverts; i++)
    {
      fVertices[i] = pts[i];
    }
  }
  Poly (const Poly& p) : fCulled(p.fCulled), fNVerts (p.fNVerts), fColor(p.fColor), fSurfaceInfo(p.fSurfaceInfo), fTextureMap(p.fTextureMap), fFakeShadow(p.fFakeShadow)
  {
    fVertices = new Vertex[p.fNVerts];
    
    for (int i = 0; i < p.fNVerts; i++)
    {
      fVertices[i] = p.fVertices[i];
    }
  }

  ~Poly();

  bool Culled() {return fCulled;}

  void Set(double *x, double *y, double *z, int nverts);
  void Set(const Vertex *pts, int nverts);

  void SetColor(double r, double g, double b);
  void SetColor(const RGBColor& rgb);
  void SetSurfaceInfo(const SurfaceInfo& info);

  void SetTextureMap(TextureMap *map);
  void SetFakeShadow(const FakeShadow& s);

  SurfaceInfo GetSurfaceInfo() const;
  RGBColor GetColor() const;

  bool ContainsVertex(int id) const;
  Vector ComputeSurfaceNormal();
  void ComputeVertexColors();
  void SetVertexNormal(int id, const Vector& normal);

  void Render(wxDC& dc);
  void MakeShadow(const Vector& lightPos);
  void ApplyMatrix(const Matrix& M);
  void CameraTransform(const Camera& C);
  void PerspectiveTransform(const Camera& C);
  void WindowTransform(const Camera& C);
  bool Cull(const Vector& V);
  bool Cull();

  friend Poly MakeShadow(const Poly& s, const Vector& lightPos);
  friend Poly ApplyMatrix(const Poly& s, const Matrix& M);
  friend Poly CameraTransform(const Poly& s, const Camera& C);
  friend Poly PerspectiveTransform(const Poly& s, const Camera& C);
  friend Poly WindowTransform(const Poly& s, const Camera& C);
  friend Poly Cull(const Poly& s, const Vector& V);
  friend Poly Cull(const Poly& s);
  friend void Render(Poly& s, wxDC& dc);

  Poly& operator=(const Poly& p);
  friend ostream& operator<<(ostream& s, const Poly& a);
};

#endif
