#ifndef _SURFACEINFO_H_
#define _SURFACEINFO_H_

#include "RGBColor.h"
#include "Vector.h"

class Point;

class SurfaceInfo
{
protected:

  RGBColor fI;
  RGBColor fOd;
  RGBColor fOs;
  double fKa;
  double fKd;
  double fKs;
  double fNs;

public:
  SurfaceInfo() : fI(0.0), fOd(0.0), fOs(0.0), fKa(1.0), fKd(1.0), fKs(1.0), fNs(0.0) {}
  SurfaceInfo(const SurfaceInfo& i) : fI(i.fI), fOd(i.fOd), fOs(i.fOs), fKa(i.fKa), fKd(i.fKd), fKs(i.fKs), fNs(i.fNs) {}

  void SetObjectDiffuseColor(const RGBColor& c);
  void SetObjectDiffuseColor(double r, double g, double b);
  void SetObjectSpecularColor(const RGBColor& c);
  void SetObjectSpecularColor(double r, double g, double b);
  void SetAmbientCoefficient(double ka);
  void SetDiffuseCoefficient(double kd);
  void SetSpecularCoefficient(double ks);
  void SetSpecularParameter(double n);

  RGBColor GetIntensity() const;

  //To make things easier with multiplying colors
  double& operator[](int _i_)
  {
    if (_i_ == 0)
      return fI[0];
    else if (_i_ == 1)
      return fI[1];
    else
      return fI[2];
  }

  const double& operator[](int _i_) const
  {
    if (_i_ == 0)
      return fI[0];
    else if (_i_ == 1)
      return fI[1];
    else
      return fI[2];
  }

  void ComputeIntensity (const Point& intersect, const Vector& normal);

  friend Vector HalfwayVector(const Vector& light, const Vector& v);

  SurfaceInfo& operator=(const SurfaceInfo& s);
};

#endif
