#ifndef _SURFACE_INFO_INL_
#define _SURFACE_INFO_INL_

#include "SurfaceInfo.h"
#include "Vector.inl"
#include "RGBColor.inl"

inline SurfaceInfo& SurfaceInfo::operator=(const SurfaceInfo& s)
{
  fI = s.fI;
  fOd = s.fOd;
  fOs = s.fOs;
  fKa = s.fKa;
  fKd = s.fKd;
  fKs = s.fKs;
  fNs = s.fNs;

  return *this;
}

inline void SurfaceInfo::SetObjectDiffuseColor(const RGBColor& c)
{
  fOd = c;

  //Default surface color is the diffuse reflectance color
  fI = c;
}

inline void SurfaceInfo::SetObjectDiffuseColor(double r, double g, double b)
{
  fOd[0] = r;
  fOd[1] = g;
  fOd[2] = b;

  //Default surface color is the diffuse reflectance color
  fI[0] = r;
  fI[1] = g;
  fI[2] = b;
}

inline void SurfaceInfo::SetObjectSpecularColor(const RGBColor& c)
{
  fOs = c;
}

inline void SurfaceInfo::SetObjectSpecularColor(double r, double g, double b)
{
  fOs[0] = r;
  fOs[1] = g;
  fOs[2] = b;
}

inline void SurfaceInfo::SetAmbientCoefficient(double ka)
{
  fKa = ka;
}

inline void SurfaceInfo::SetDiffuseCoefficient(double kd)
{
  fKd = kd;
}

inline void SurfaceInfo::SetSpecularCoefficient(double ks)
{
  fKs = ks;
}

inline void SurfaceInfo::SetSpecularParameter(double n)
{
  fNs = n;
}

inline RGBColor SurfaceInfo::GetIntensity() const
{
  return fI;
}

//Calculates orienation of maximum highlight for specular light (halfway vector)
inline Vector HalfwayVector(const Vector& light, const Vector& v)
{
  Vector ret = light + v;
  return Normalize(ret);
}

#endif
