#ifndef _VECTOR_H_
#define _VECTOR_H_

#include <math.h>

class ostream;
class Matrix;

class Vector
{
protected:

  double fX;
  double fY;
  double fZ;

public:

  Vector() : fX(0.0), fY(0.0), fZ(0.0) {}
  Vector(double _x_, double _y_, double _z_) : fX(_x_), fY(_y_), fZ(_z_) {}
  Vector(double _x_) : fX(_x_), fY(_x_), fZ(_x_) {}
  Vector(const Vector& v) : fX(v.fX), fY(v.fY), fZ(v.fZ) {}

  double& operator[](int _i_)
  {
    if (_i_ == 0)
      return fX;
    else if (_i_ == 1)
      return fY;
    else
      return fZ;
  }

  const double& operator[](int _i_) const
  {
    if (_i_ == 0)
      return fX;
    else if (_i_ == 1)
      return fY;
    else
      return fZ;
  }

  friend double Magnitude(const Vector& v);
  friend double DotProduct(const Vector& a, const Vector& b);
  friend Vector CrossProduct(const Vector& a, const Vector& b);
  friend Vector Normalize(const Vector& v);

  //Unary operators
  Vector operator+() const;
  Vector operator-() const;

  //Assignment operators
  Vector& operator=(const Vector& v);
  Vector& operator+=(const Vector& v);
  Vector& operator-=(const Vector& v);
  Vector& operator*=(const Vector& v);
  Vector& operator*=(double v);
  Vector& operator/=(double v);

  //Binary operators
  friend Vector operator+(const Vector& a, const Vector& b);
  friend Vector operator-(const Vector& a, const Vector& b);
  friend Vector operator*(const Vector& a, const Vector& b);
  friend Vector operator*(const Vector& a, double b);
  friend Vector operator*(double a, const Vector& b);
  friend Vector operator/(const Vector& a, const Vector& b);
  friend Vector operator/(const Vector& a, double b);

  //This one is in the matrix inline file
  friend Vector operator*(const Matrix& M, const Vector& v);

  friend int operator==(const Vector& a, const Vector& b);
  friend int operator<(const Vector& a, const Vector& b);
  friend ostream& operator<<(const ostream& s, const Vector& a);
};

#endif
