#ifndef _VECTRO_INL_
#define _VECTRO_INL_

#include "Vector.h"

inline double Magnitude(const Vector& v)
{
  return sqrt ((v.fX * v.fX) + (v.fY * v.fY) + (v.fZ * v.fZ));
}

inline Vector Normalize(const Vector& v)
{
  double mag = Magnitude(v);

  //If the vector is not all zero, divide
  return (mag != 0) ? v/mag : v;
}

inline double DotProduct(const Vector& a, const Vector& b)
{
  return (a.fX * b.fX) + (a.fY * b.fY) + (a.fZ * b.fZ);
}

inline Vector CrossProduct(const Vector& a, const Vector& b)
{
  return Vector(a.fY * b.fZ - a.fZ * b.fY,
                a.fZ * b.fX - a.fX * b.fZ,
                a.fX * b.fY - a.fY * b.fX);
}

inline Vector Vector::operator+() const
{
  return *this;
}

inline Vector Vector::operator-() const
{
  return Vector(-fX, -fY, -fZ);
}

inline Vector& Vector::operator=(const Vector& v)
{
  fX = v.fX;
  fY = v.fY;
  fZ = v.fZ;

  return *this;
}

inline Vector& Vector::operator+=(const Vector& v)
{
  fX += v.fX;
  fY += v.fY;
  fZ += v.fZ;

  return *this;
}

inline Vector& Vector::operator-=(const Vector& v)
{
  fX -= v.fX;
  fY -= v.fY;
  fZ -= v.fZ;

  return *this;
}

inline Vector& Vector::operator*=(const Vector& v)
{
  fX *= v.fX;
  fY *= v.fY;
  fZ *= v.fZ;

  return *this;
}

inline Vector& Vector::operator*=(double v)
{
  fX *= v;
  fY *= v;
  fZ *= v;

  return *this;
}

inline Vector& Vector::operator/=(double v)
{
  fX /= v;
  fY /= v;
  fZ /= v;

  return *this;
}

inline Vector operator+(const Vector& a, const Vector& b)
{
  return Vector(a.fX + b.fX, a.fY + b.fY, a.fZ + b.fZ);
}

inline Vector operator-(const Vector& a, const Vector& b)
{
  return Vector(a.fX - b.fX, a.fY - b.fY, a.fZ - b.fZ);
}

inline Vector operator*(const Vector& a, const Vector& b)
{
  return Vector(a.fX * b.fX, a.fY * b.fY, a.fZ * b.fZ);
}

inline Vector operator*(const Vector& a, double b)
{
  return Vector(a.fX * b, a.fY * b, a.fZ * b);
}

inline Vector operator*(double a, const Vector& b)
{
  return b * a;
}

inline Vector operator/(const Vector& a, const Vector& b)
{
  return Vector(a.fX / b.fX, a.fY / b.fY, a.fZ / b.fZ);
}

inline Vector operator/(const Vector& a, double b)
{
  return Vector(a.fX / b, a.fY / b, a.fZ / b);
}

inline int operator==(const Vector& a, const Vector& b)
{
  return (a.fX == b.fX) && (a.fY == b.fY) && (a.fZ == b.fZ);
}

inline int operator<(const Vector& a, const Vector& b)
{
  return (a.fX < b.fX) && (a.fY < b.fY) && (a.fZ < b.fZ);
}

#endif
