#ifndef _VERTEX_H_
#define _VERTEX_H_

#include <math.h>
#include "RGBColor.h"
#include "Point.h"

class ostream;
class Matrix;
class Camera;
class FakeShadow;

class Vertex
{
protected:

  int fID;
  Point fPoint;
  Point fWC;
  Vector fNormal;
  RGBColor fColor;

public:

  Vertex() : fPoint(0.0), fID(-1), fColor(0.0), fNormal(0.0), fWC(0.0) {}
  Vertex(double _x_, double _y_, double _z_, double _h_ = 1.0) : fPoint(_x_, _y_, _z_, _h_), fWC(_x_, _y_, _z_, _h_), fColor(0.0), fNormal(0.0), fID(-1) {}
  Vertex(double _x_) : fPoint(_x_), fWC(_x_), fColor(0.0), fNormal(0.0), fID(-1) {}
  Vertex(const Point& p, const Vector& v, const RGBColor& r) : fPoint(p), fWC(p), fColor(r), fNormal(v), fID(-1) {}
  Vertex(const Vertex& v) : fPoint(v.fPoint), fWC(v.fWC), fColor(v.fColor), fID(v.fID), fNormal(v.fNormal) {}

  void Set(double x, double y, double z, double h = 1.0);
  void Set(const Vertex& p);

  void SetID(int id);
  int GetID() const;

  void SetPoint(const Point& p);
  Point GetPoint() const;

  void SetWC(const Point& wc);
  Point GetWC() const;

  void SetColor(const RGBColor& color);
  void SetColor(double r, double g, double b);
  RGBColor GetColor() const;

  double GetX() const;
  double GetY() const;
  double GetZ() const;
  double GetH() const;

  void SetNormal(const Vector& normal);
  Vector GetNormal() const;

  Vertex& operator=(const Vertex& p);

  friend ostream& operator<<(ostream& s, const Vertex& a);

  void ApplyShadow (const FakeShadow& F);
  void ApplyMatrix(const Matrix& M);
  void CameraTransform(const Camera& C);
  void PerspectiveTransform(const Camera& C);
  void WindowTransform(const Camera& C);

  friend Vertex ApplyShadow(const Vertex& s, const FakeShadow& F);
  friend Vertex ApplyMatrix(const Vertex& s, const Matrix& M);
  friend Vertex CameraTransform(const Vertex& s, const Camera& C);
  friend Vertex PerspectiveTransform(const Vertex& s, const Camera& C);
  friend Vertex WindowTransform(const Vertex& s, const Camera& C);
};

#endif
