#ifndef _VERTEX_INL_
#define _VERTEX_INL_

#include "Vertex.h"
#include "Matrix.h"
#include "Camera.h"
#include "FakeShadow.h"

#include "Matrix.inl"
#include "Camera.inl"
#include "FakeShadow.inl"
#include "RGBColor.inl"
#include "Point.inl"

inline double Vertex::GetX() const
{
  return fPoint.GetX();
}

inline double Vertex::GetY() const
{
  return fPoint.GetY();
}

inline double Vertex::GetZ() const
{
  return fPoint.GetZ();
}

inline double Vertex::GetH() const
{
  return fPoint.GetH();
}

inline void Vertex::Set(double x, double y, double z, double h)
{
  fPoint.Set(x, y, z, h);
  fWC.Set(x, y, z, h);
}

inline void Vertex::Set(const Vertex& p)
{
  fPoint = p.fPoint;
  fWC = p.fWC;
  fColor = p.fColor;
  fID = p.fID;
  fNormal = p.fNormal;
}

inline void Vertex::SetID(int id)
{
  fID = id;
}

inline int Vertex::GetID() const
{
  return fID;
}

inline void Vertex::SetPoint(const Point& p)
{
  fPoint = p;
}

inline Point Vertex::GetPoint() const
{
  return fPoint;
}

inline void Vertex::SetWC(const Point& wc)
{
  fWC = wc;
}

inline Point Vertex::GetWC() const
{
  return fWC;
}

inline void Vertex::SetColor(const RGBColor& color)
{
  fColor = color;
}

inline void Vertex::SetColor(double r, double g, double b)
{
  fColor[0] = r;
  fColor[1] = g;
  fColor[2] = b;
}

inline RGBColor Vertex::GetColor() const
{
  return fColor;
}

inline void Vertex::SetNormal(const Vector& normal)
{
  fNormal = normal;
}

inline Vector Vertex::GetNormal() const
{
  return fNormal;
}

inline Vertex& Vertex::operator=(const Vertex& v)
{
  fPoint = v.fPoint;
  fWC = v.fWC;
  fColor = v.fColor;
  fID = v.fID;
  fNormal = v.fNormal;

  return *this;
}

inline void Vertex::ApplyShadow (const FakeShadow& F)
{
  fPoint = ::ApplyShadow(F, fPoint);
  fWC = fPoint;  //Done in World space
}

inline void Vertex::ApplyMatrix(const Matrix& M)
{
  //Apply transformation to each point
  fPoint = M * fPoint;
  fWC = fPoint;
}

inline void Vertex::CameraTransform(const Camera& C)
{
  //Store wc for color calculations
  fWC = fPoint;
  fPoint = ApplyCamera(C, fPoint);
}

inline void Vertex::PerspectiveTransform(const Camera& C)
{
  fPoint = ApplyPerspective(C, fPoint);
}

inline void Vertex::WindowTransform(const Camera& C)
{
  fPoint = ApplyWindow(C, fPoint);
}

inline Vertex ApplyShadow(const Vertex& s, const FakeShadow& F)
{
  Vertex apply = s;
  apply.ApplyShadow(F);
  return apply;
}

inline Vertex ApplyMatrix(const Vertex& s, const Matrix& M)
{
  Vertex apply = s;
  apply.ApplyMatrix(M);
  return apply;
}

inline Vertex CameraTransform(const Vertex& s, const Camera& C)
{
  Vertex apply = s;
  apply.CameraTransform(C);
  return apply;
}

inline Vertex PerspectiveTransform(const Vertex& s, const Camera& C)
{
  Vertex apply = s;
  apply.PerspectiveTransform(C);
  return apply;
}

inline Vertex WindowTransform(const Vertex& s, const Camera& C)
{
  Vertex apply = s;
  apply.WindowTransform(C);
  return apply;
}

#endif
