#include "wx/list.h"

#include "Canvas.h"
#include "Environment.inl"

BEGIN_EVENT_TABLE(Canvas, wxScrolledWindow)
    EVT_SIZE(Canvas::OnSize)
END_EVENT_TABLE()

Canvas::Canvas(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style, const wxString& name)
: wxScrolledWindow(parent, id, pos, size, style, name),
  fBackBuffer( NULL ),
  fRedo(true)
{
  fEnvironment.ReadEnvironmentFile("env.dat");
  fEnvironment.ReadObjectFiles("objects.d");
  fEnvironment.ReadCameraFile("camera.dat");
  fEnvironment.ReadAmbientLightFile("ambientlight.dat");
  fEnvironment.ReadPointLightFile("pointlights.dat");
  if (Environment::SHADOWTYPE == Environment::FAKESHADOW)
    fEnvironment.ReadShadowFile("shadowplane.dat");
}

Canvas::~Canvas()
{
  if (fBackBuffer)
    delete fBackBuffer;

}

void Canvas::OnSize(wxSizeEvent &event)
{
  int i;
  int width, height;
  GetSize(&width, &height);

  Environment::WIDTH = width;
  Environment::HEIGHT = height;
  Environment::fDepthBuffer.SetSize(width, height);

  for (i = 0; i < Environment::fNumShadowBuffers; i++)
  {
    Environment::fShadowBuffers[i].SetSize(width, height);
  }

  if (fBackBuffer)
    delete fBackBuffer;
  fBackBuffer = new wxBitmap( width, height );

  fRedo = true;
}

void Canvas::OnDraw(wxDC& dc)
{
  wxMemoryDC memoryDC;

  memoryDC.SelectObject(*fBackBuffer);

  if (fRedo)
  {
    fEnvironment.Render( memoryDC );
    fRedo = false;
  }

  //Swap buffer
  dc.Blit(0, 0, fBackBuffer->GetWidth(), fBackBuffer->GetHeight(), &memoryDC, 0, 0, wxCOPY);
  memoryDC.SelectObject(wxNullBitmap);
}
