#include "wx/wxprec.h"

#include <iostream.h>
#include <iomanip.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>

#include "Model.h"
#include "Vertex.h"
#include "Poly.h"
#include "Point.h"
#include "SphericalTextureMap.h"
#include "CylindricalTextureMap.h"
#include "FakeShadow.h"

#include "Model.inl"
#include "Vertex.inl"
#include "Poly.inl"
#include "Point.inl"
#include "FakeShadow.inl"

#ifdef WIN32
#define strcasecmp _stricmp
#endif

Model::~Model()
{
  if (fPolys)
    delete [] fPolys;
  if (fVertices)
    delete [] fVertices;
  if (fTextureMap)
    delete fTextureMap;
}

void Model::ReadObjectFile(const char *file)
{
  FILE *objects;
  char line[1024];
  char *ptr;
  char *token;
  int total, corner, i, j;
  double x, y, z;
  objects = fopen(file, "r");

  fgets(line, sizeof(line), objects);
  ptr = line;
  while(!isdigit(*ptr))
    ptr++;

  token = strtok(ptr, " \t");
  fNumVerts = atoi(token);
  fVertices = new Vertex[fNumVerts];

  token = strtok(NULL, " \t");
  fNumPolys = atoi(token);
  fPolys = new Poly[fNumPolys];

  //Read vertices
  for (i = 0; i < fNumVerts; i++)
  {
    fgets(line, sizeof(line), objects);
    token = strtok(line, " \t");
    x = atof (token);
    token = strtok(NULL, " \t");
    y = atof (token);
    token = strtok(NULL, " \t");
    z = atof (token);
    fVertices[i].Set(x, y, z);
    fVertices[i].SetID(i);
  }

  //Read faces
  for (i = 0; i < fNumPolys; i++)
  {
    //Build references to the Vertex table for each specific face
    fgets(line, sizeof(line), objects);
    token = strtok(line, " \t");
    total = atoi(token);

    Vertex *xyz = new Vertex[total];
    for (j = 0; j < total; j++)
    {
      token = strtok(NULL, " \t");
      corner = atoi(token);
      xyz[j] = fVertices[corner-1];
    }

    //Create polygon face
    fPolys[i].Set(xyz, total);
    delete [] xyz;
  }

  fclose (objects);
}

void Model::ReadColorFile(const char *file)
{
  FILE *objects;
  char line[1024];
  char *token;
  int total, i, j;
  double r, g, b;
  objects = fopen(file, "r");

  fgets(line, sizeof(line), objects);
  total = atoi (line);

  for (i = 0; i < total; i++)
  {
    //Read the color
    fgets(line, sizeof(line), objects);
    token = strtok(line, " \t");
    r = atof(token);
    token = strtok(NULL, " \t");
    g = atof(token);
    token = strtok(NULL, " \t");
    b = atof(token);

    //Color every nth polygon
    for (j = i; j < fNumPolys; j += total)
      fPolys[j].SetColor(r, g, b);
  }

  fclose (objects);
}

void Model::ReadSurfaceInfoFile(const char *file)
{
  int i;
  double col[3];
  FILE *fp;
  char line[1024];
  char *token;
  SurfaceInfo info;

  fp = fopen(file, "r");

  //Read input from file
  fgets(line, sizeof(line), fp);
  token = strtok(line, " \t");
  col[0] = atof(token);
  token = strtok(NULL, " \t");
  col[1] = atof(token);
  token = strtok(NULL, " \t");
  col[2] = atof(token);

  info.SetObjectSpecularColor(col[0], col[1], col[2]);

  fgets(line, sizeof(line), fp);
  col[0] = atof(line);
  info.SetAmbientCoefficient(col[0]);

  fgets(line, sizeof(line), fp);
  col[0] = atof(line);
  info.SetDiffuseCoefficient(col[0]);

  fgets(line, sizeof(line), fp);
  col[0] = atof(line);
  info.SetSpecularCoefficient(col[0]);

  fgets(line, sizeof(line), fp);
  col[0] = atof(line);
  info.SetSpecularParameter(col[0]);

  for (i = 0; i < fNumPolys; i++)
  {
    //Od was specified separatley to let an object have multicolored polygons
    info.SetObjectDiffuseColor(fPolys[i].GetColor());
    fPolys[i].SetSurfaceInfo(info);
  }

  fclose(fp);
}

void Model::ReadTextureFile(const char *file)
{
  int i;
  FILE *fp;
  char line[1024];
  char *token;
  char *newline;
  SurfaceInfo info;

  fp = fopen(file, "r");

  //Read input from file
  fgets(line, sizeof(line), fp);
  newline = strchr(line, '\n');
  if (newline)
    *newline = '\0';

  if (fTextureMap)
    delete fTextureMap;
  fTextureMap = NULL;

  token = strtok(line, " \t");
  if (strcasecmp(token, "SPHERE") == 0)
    fTextureMap = new SphericalTextureMap(1.0);
  else
    fTextureMap = new CylindricalTextureMap(1.0, 2.0);

  token = strtok(NULL, " \t");
  fTextureMap->LoadTexture(token);

  for (i = 0; i < fNumPolys; i++)
  {
    fPolys[i].SetTextureMap(fTextureMap);
  }

  fclose(fp);
}

void Model::ReadNormalFile(const char *file)
{
  FILE *objects;
  char line[1024];
  char *token;
  int total, i, j;
  Vector normal;
  objects = fopen(file, "r");

  fgets(line, sizeof(line), objects);

  token = strtok(line, " \t");
  total = atoi(token);

  //Read vertices
  for (i = 0; i < total; i++)
  {
    fgets(line, sizeof(line), objects);
    token = strtok(line, " \t");
    normal[0] = atof (token);
    token = strtok(NULL, " \t");
    normal[1] = atof (token);
    token = strtok(NULL, " \t");
    normal[2] = atof (token);

    //Set it for each polygon
    for (j = 0; j < fNumPolys; j++)
    {
      fPolys[j].ContainsVertex(i);
        fPolys[j].SetVertexNormal(i, -normal);
    }
  }

  fclose (objects);
}

void Model::ReadShadowFile(const char *file)
{
  FILE *objects;
  char line[1024];
  char *token;
  int j;
  Point p[3];
  double x, y, z;
  FakeShadow shadow;

  objects = fopen(file, "r");

  //Read the points
  for (j = 0; j < 3; j++)
  {
    fgets(line, sizeof(line), objects);
    token = strtok(line, " \t");
    x = atof(token);
    token = strtok(NULL, " \t");
    y = atof(token);
    token = strtok(NULL, " \t");
    z = atof(token);
    p[j].Set(x, y, z);
  }

  shadow.FindShadowPlane(p[0], p[1], p[2]);
  for (j = 0; j < fNumPolys; j++)
    fPolys[j].SetFakeShadow(shadow);

  fclose (objects);
}

ostream& operator<<(ostream& s, const Model& a)
{
  int i;

  for ( i = 0 ; i < a.fNumPolys; i++)
  {
    s << setprecision(2) << setw(10) << a.fPolys[i];
    s << '\n';
  }

  return s;
}
