#include "wx/wxprec.h"
#include "ScanFill.h"
#include "Environment.h"
#include "SurfaceInfo.h"
#include "TextureMap.h"

#include "Camera.inl"
#include "Environment.inl"
#include "SurfaceInfo.inl"
#include "DepthBuffer.inl"
#include "RGBColor.inl"
#include "Point.inl"

/* Inserts edge into list in order of increasing xIntersect field. */
void ScanFill::_InsertEdge (Edge * list, Edge * edge)
{
  Edge * p, * q = list;

  p = q->next;
  while (p != NULL) {
    if (edge->xIntersect < p->xIntersect)
      p = NULL;
    else {
      q = p;
      p = p->next;
    }
  }
  edge->next = q->next;
  q->next = edge;
}

/* For an index, return y-coordinate of next nonhorizontal line */
int ScanFill::_YNext (int k, int cnt, ScanPoint * pts)
{
  int j;

  if ((k+1) > (cnt-1))
    j = 0;
  else
    j = k + 1;
  while (pts[k].y == pts[j].y)
    if ((j+1) > (cnt-1))
      j = 0;
    else
      j++;
  return (pts[j].y);
}

/* Store lower-y coordinate and inverse slope for each edge.  Adjust
   and store upper-y coordinate for edges that are the lower member
   of a monotically increasing or decreasing pair of edges */
void ScanFill::_MakeEdgeRec (ScanPoint lower, ScanPoint upper, int yComp, Edge * edge, Edge * edges[])
{
  edge->dxPerScan = (double) (upper.x - lower.x) / (upper.y - lower.y);
  edge->xIntersect = lower.x;
  if (upper.y < yComp)
    edge->yUpper = upper.y - 1;
  else
    edge->yUpper = upper.y;

  edge->dzPerScan = (double) (upper.z - lower.z) / (upper.y - lower.y);
  edge->zValue = lower.z;

  if (Environment::RENDERTYPE == Environment::GOURARD)
  {
    edge->diPerScan = (upper.i - lower.i) / ((double)(upper.y - lower.y));
    edge->iValue = lower.i;
  }
  else if (Environment::RENDERTYPE == Environment::PHONG)
  {
    edge->dnPerScan = (upper.n - lower.n) / ((double)(upper.y - lower.y));
    edge->nValue = lower.n;

    edge->dwcPerScan[0] = (upper.wc[0] - lower.wc[0]) / ((double)(upper.y - lower.y));
    edge->dwcPerScan[1] = (upper.wc[1] - lower.wc[1]) / ((double)(upper.y - lower.y));
    edge->dwcPerScan[2] = (upper.wc[2] - lower.wc[2]) / ((double)(upper.y - lower.y));
    edge->wcValue.Set(lower.wc);
  }

  /* Old interpolating code
  edge->lowerX = lower.x;
  edge->lowerY = lower.y;
  edge->lowerZ = lower.z;
  edge->lowerI = lower.i;
  edge->lowerN = lower.n;
  edge->lowerWC = lower.wc;
  edge->upperX = upper.x;
  edge->upperY = upper.y;
  edge->upperZ = upper.z;
  edge->upperI = upper.i;
  edge->upperN = upper.n;
  edge->upperWC = upper.wc;
*/

  _InsertEdge (edges[lower.y], edge);
}

void ScanFill::_BuildEdgeList (int cnt, ScanPoint * pts, Edge * edges[])
{
  Edge * edge;
  ScanPoint v1, v2;
  int i, yPrev = pts[cnt - 2].y;
  
  v1.x = pts[cnt-1].x; v1.y = pts[cnt-1].y;
  v1.z = pts[cnt-1].z; v1.i = pts[cnt-1].i;
  v1.n = pts[cnt-1].n; v1.wc = pts[cnt-1].wc;
  for (i=0; i<cnt; i++) {
    v2 = pts[i];
    if (v1.y != v2.y) {                /* nonhorizontal line */
      edge = new Edge;
      if (v1.y < v2.y)                 /* up-going edge      */
	_MakeEdgeRec (v1, v2, _YNext (i, cnt, pts), edge, edges);
      else                             /* down-going edge    */
	_MakeEdgeRec (v2, v1, yPrev, edge, edges);
    }
    yPrev = v1.y;
    v1 = v2;
  }
}

void ScanFill::_BuildActiveList (int scan, Edge * active, Edge * edges[])
{
  Edge * p, * q;

  p = edges[scan]->next;
  while (p) {
    q = p->next;
    _InsertEdge (active, p);
    p = q;
  }
}

void ScanFill::_FillScan (wxDC &dc, int scan, Edge * active)
{
  Edge * p1, * p2;
  int j, xa, xb;
  double z;
  double zInc;
  RGBColor i;
  RGBColor iInc;
  Vector n;
  Vector nInc;
  Point wc;
  Point wcInc;

  p1 = active->next;
  while (p1 && p1->next)
  {
    p2 = p1->next;

    z = p1->zValue;
    zInc = (double) (p2->zValue - p1->zValue) / (p2->xIntersect - p1->xIntersect);

    //If its Flat just set the color to the shape color
    //If Gourard setup i increments
    //If phong set up normal increments
    if (Environment::RENDERTYPE == Environment::FLAT)
    {
      i = fCurrentSurfaceInfo->GetIntensity();
    }
    else if (Environment::RENDERTYPE == Environment::GOURARD)
    {
      i = p1->iValue;
      iInc = (p2->iValue - p1->iValue) / (p2->xIntersect - p1->xIntersect);
    }
    else if (Environment::RENDERTYPE == Environment::PHONG)
    {
      n = p1->nValue;
      nInc = (p2->nValue - p1->nValue) / (p2->xIntersect - p1->xIntersect);

      wc = p1->wcValue;
      wcInc[0] = (p2->wcValue[0] - p1->wcValue[0]) / (p2->xIntersect - p1->xIntersect);
      wcInc[1] = (p2->wcValue[1] - p1->wcValue[1]) / (p2->xIntersect - p1->xIntersect);
      wcInc[2] = (p2->wcValue[2] - p1->wcValue[2]) / (p2->xIntersect - p1->xIntersect);
    }

    xa = (int)p1->xIntersect;
    xb = (int)p2->xIntersect;
    for (j = xa; j <= xb; j++)
    {
      //Possibly processing a shadow buffer
      if (Environment::fCurrentShadowBuffer != -1)
      {
        //Just put in the values for the shadow
        Environment::fShadowBuffers[Environment::fCurrentShadowBuffer].DepthTest(j, scan, z);
      }
      else
      {
  //      z = _ComputeZ(p1, p2, i, scan);
        if (Environment::fDepthBuffer.DepthTest(j, scan, z))
        {
          //Set the color
  //        RGBColor rgb = _ComputeColor((int) i, scan, z, p1, p2);
  //        wxColour col(255 * rgb[0], 255 * rgb[1], 255 * rgb[2]);

          if (Environment::RENDERTYPE == Environment::GOURARD)
          {
            i = NormalizeColor(i);
          }
          else if (Environment::RENDERTYPE == Environment::PHONG)
          {
            //Must be renormalized with each calculation
            n = Normalize(n);

            if (fCurrentTextureMap)
            {
              RGBColor col = fCurrentTextureMap->Map3DTo2D(n);
              fCurrentSurfaceInfo->SetObjectDiffuseColor(NormalizeColor(col));
            }

   	        //Calculate color
 	          fCurrentSurfaceInfo->ComputeIntensity (wc, n);
   	        i = fCurrentSurfaceInfo->GetIntensity();
    	  
   	        //Clamp
  	        i = NormalizeColor(i);
          }

          //Check to see if in shadow
          if (Environment::SHADOWTYPE == Environment::ZBUFFERSHADOW)
          {
            int k;
            double shadowdepth;
            for (k = 0; k < Environment::fNumShadowBuffers; k++)
            {
              shadowdepth = Environment::fShadowBuffers[k].GetDepth(j, scan);

              //If behind an object in shadow buffer, adjust color
              //Offset shadow depth so it doesn't draw over its creating surface
              if (z > (shadowdepth + 0.01))
              {
                //Darken it by distance between (closer is darker)
                i[0] *= 0.4 + z - shadowdepth;
                i[1] *= 0.4 + z - shadowdepth;
                i[2] *= 0.4 + z - shadowdepth;
              }
            }
          }

          wxColour col(255 * i[0], 255 * i[1], 255 * i[2]);

          wxPen colpen(col, 1, wxSOLID);
          dc.SetPen(colpen);

          dc.DrawPoint(j, scan);
        }
      }
      z += zInc;

      if (Environment::RENDERTYPE == Environment::GOURARD)
      {
        i += iInc;
      }
      else if (Environment::RENDERTYPE == Environment::PHONG)
      {
        n += nInc;
        wc[0] += wcInc[0];
        wc[1] += wcInc[1];
        wc[2] += wcInc[2];
      }
    }
    p1 = p2->next;
  }
}

void ScanFill::_DeleteAfter (Edge * q)
{
  Edge * p = q->next;

  q->next = p->next;
  delete p;
}

/* Delete completed edges. Update 'xIntersect' field for others */
void ScanFill::_UpdateActiveList (int scan, Edge * active)
{
  Edge * q = active, * p = active->next;

  while (p) 
    if (scan >= p->yUpper) {
      p = p->next;
      _DeleteAfter (q);
    }
    else {
      p->xIntersect = p->xIntersect + p->dxPerScan;
      p->zValue = p->zValue + p->dzPerScan;

      if (Environment::RENDERTYPE == Environment::GOURARD)
      {
        p->iValue += p->diPerScan;
      }
      else if (Environment::RENDERTYPE == Environment::PHONG)
      {
        p->nValue += p->dnPerScan;

        p->wcValue[0] += p->dwcPerScan[0];
        p->wcValue[1] += p->dwcPerScan[1];
        p->wcValue[2] += p->dwcPerScan[2];
      }

      q = p;
      p = p->next;
    }
}

void ScanFill::_ResortActiveList (Edge * active)
{
  Edge * q, * p = active->next;

  active->next = NULL;
  while (p) {
    q = p->next;
    _InsertEdge (active, p);
    p = q;
  }
}

void ScanFill::Scan (wxDC& dc, int cnt, ScanPoint * pts)
{
  Edge ** edges, * active;
  int i, scan;

  edges = new Edge*[Environment::HEIGHT];

  for (i=0; i<Environment::HEIGHT; i++)
  {
    edges[i] = new Edge;
    edges[i]->next = NULL;
  }
  _BuildEdgeList (cnt, pts, edges);
  active = new Edge;
  active->next = NULL;

  for (scan=0; scan<Environment::HEIGHT; scan++)
  {
    _BuildActiveList (scan, active, edges);
    if (active->next) {
      _FillScan (dc, scan, active);
      _UpdateActiveList (scan, active);
      _ResortActiveList (active);
    }
  }
  /* Free edge records that have been malloc'ed ... */

  while(active->next)
    _DeleteAfter(active);
  delete active;

  for (i=0; i<Environment::HEIGHT; i++)
  {
    delete edges[i];
  }
  delete [] edges;
}

/*
double ScanFill::_ComputeZ (Edge *e1, Edge *e2, int xs, int ys)
{
	double xa = (double)e1->xIntersect;
 	double xb = (double)e2->xIntersect;
 	double y1 = (double)e1->lowerY;
 	double y2 = (double)e1->upperY;
 	double y3 = (double)e2->lowerY;
 	double y4 = (double)e2->upperY;
 	double z1 = (double)e1->lowerZ;
 	double z2 = (double)e1->upperZ;
 	double z3 = (double)e2->lowerZ;
 	double z4 = (double)e2->upperZ;
  double za;
  double zb;
  double zs;

  za = z1 - ((z1 - z2) * ((y1 - (double)ys)/(y1 - y2)));
  zb = z3 - ((z3 - z4) * ((y3 - (double)ys)/(y3 - y4)));
  zs = zb - ((zb - za) * ((xb - (double)xs)/(xb - xa)));

  return zs;
}

//Functinon to calculate color at specific point
RGBColor ScanFill::_ComputeColor (int xs, int ys, double zs, Edge *e1, Edge *e2)
{
  RGBColor Is;

  if (Environment::RENDERTYPE == Environment::FLAT)
  {
    //Set color to color of upper left hand corner (lower is higher in device coordinates)
    Is = fCurrentSurfaceInfo->GetIntensity();
  }
  else if (Environment::RENDERTYPE == Environment::GOURARD)
  {
    int xa = (int)e1->xIntersect;
    int xb = (int)e2->xIntersect;
    int x1 = e1->lowerX;
    int x2 = e1->upperX;
    int x3 = e2->lowerX;
    int x4 = e2->upperX;
    int y1 = e1->lowerY;
    int y2 = e1->upperY;
    int y3 = e2->lowerY;
    int y4 = e2->upperY;
    RGBColor I1 = e1->lowerI; 
    RGBColor I2 = e1->upperI; 
    RGBColor I3 = e2->lowerI; 
    RGBColor I4 = e2->upperI; 
    RGBColor Ia;
    RGBColor Ib;
      
    //Interpolate color at point
   	Ia = (1.0/((double)y1 - y2)) * (((double)(ys - y2) * I1) + ((double)(y1 - ys) * I2));
   	Ib = (1.0/((double)y3 - y4)) * (((double)(ys - y4) * I3) + ((double)(y3 - ys) * I4));
   	Is = (1.0/((double)xb - xa)) * (((double)(xb - xs) * Ia) + ((double)(xs - xa) * Ib));

    //Clamp
    Is = NormalizeColor(Is);
  }
	else if (Environment::RENDERTYPE == Environment::PHONG)
  {
    int xa = (int)e1->xIntersect;
    int xb = (int)e2->xIntersect;
    int x1 = e1->lowerX;
    int x2 = e1->upperX;
    int x3 = e2->lowerX;
    int x4 = e2->upperX;
    int y1 = e1->lowerY;
    int y2 = e1->upperY;
    int y3 = e2->lowerY;
    int y4 = e2->upperY;
    Vector N1 = e1->lowerN; 
    Vector N2 = e1->upperN; 
    Vector N3 = e2->lowerN; 
    Vector N4 = e2->upperN; 
    Vector Na;
    Vector Nb;
    Vector Ns;
    Vector Inter1(e1->lowerWC.GetX(), e1->lowerWC.GetY(), e1->lowerWC.GetZ());
    Vector Inter2(e1->upperWC.GetX(), e1->upperWC.GetY(), e1->upperWC.GetZ());
    Vector Inter3(e2->lowerWC.GetX(), e2->lowerWC.GetY(), e2->lowerWC.GetZ());
    Vector Inter4(e2->upperWC.GetX(), e2->upperWC.GetY(), e2->upperWC.GetZ());
    Vector InterA;
    Vector InterB;
    Vector InterS;

    //Interpolate WC
    //Interpolate normal at point
   	InterA = (1.0/((double)y1 - y2)) * (((double)(ys - y2) * Inter1) + ((double)(y1 - ys) * Inter2));
   	InterB = (1.0/((double)y3 - y4)) * (((double)(ys - y4) * Inter3) + ((double)(y3 - ys) * Inter4));
   	InterS = (1.0/((double)xb - xa)) * (((double)(xb - xs) * InterA) + ((double)(xs - xa) * InterB));
    Point intersect(InterS[0], InterS[1], InterS[2]);

    //Interpolate normal at point
   	Na = (1.0/((double)y1 - y2)) * (((double)(ys - y2) * N1) + ((double)(y1 - ys) * N2));
		Na = Normalize (Na);

   	Nb = (1.0/((double)y3 - y4)) * (((double)(ys - y4) * N3) + ((double)(y3 - ys) * N4));
		Nb = Normalize (Nb);

   	Ns = (1.0/((double)xb - xa)) * (((double)(xb - xs) * Na) + ((double)(xs - xa) * Nb));
   	Ns = Normalize (Ns);
    	
    if (fCurrentTextureMap)
    {
      RGBColor col = fCurrentTextureMap->Map3DTo2D(Ns);
      fCurrentSurfaceInfo->SetObjectDiffuseColor(NormalizeColor(col));
    }
   	//Calculate Is
 	  fCurrentSurfaceInfo->ComputeIntensity (intersect, Ns);
   	Is = fCurrentSurfaceInfo->GetIntensity();
    	
   	//Clamp
  	Is = NormalizeColor(Is);
  }
  return Is;
}
*/
