#include <math.h>
#include "Point.h"
#include "SurfaceInfo.h"
#include "Environment.h"

#include "SurfaceInfo.inl"
#include "Environment.inl"
#include "AmbientLight.inl"
#include "PointLight.inl"
#include "Point.inl"

//Calculate the color at the curent point
void SurfaceInfo::ComputeIntensity (const Point& inter, const Vector& normal)
{
  int i;
  int j;
  int total = 0;
  double NL = 0.0;
  double NH = 0.0;
  RGBColor ambient(0.0);
  RGBColor partialDiff(0.0);
  RGBColor partialSpec(0.0);
  RGBColor sum1(0.0);
  RGBColor sum2(0.0);
  Vector lightDir(0.0);
  Vector halfway(0.0);
  Vector intersect(inter.GetX(), inter.GetY(), inter.GetZ());
  Vector viewDir = Normalize(Environment::fCamera.GetCP() - intersect);

  if (Environment::LIGHTSCENE == false)
  {
    fI = NormalizeColor(fOd);
  }
  else
  {
    //Extract the current light and make the calculation
    sum1 += fKa * (Environment::fAmbientLight.GetIntensity() * fOd);
    
    total = Environment::fNumPointLights;
    for (i = 0; i < total; i++)
    {
      //calculate direction to light
      lightDir = Normalize (Environment::fPointLights[i].GetPosition() - intersect);

      //Calculate N * L for diffuse
      NL = DotProduct (normal, lightDir);

      //Calculate H for specular
      halfway = HalfwayVector (lightDir, viewDir);
      
      //Calculate N * H for specular
      NH = DotProduct (normal, halfway);
      NH = pow (NH, fNs);

      //Now sum it all up
      if (NL >= 0)
      {
        partialDiff = (fKd * NL) * fOd;
      }
      else
      {
        for (j = 0; j < 3; j++)
          partialDiff[j] = 0.0;
      }

      if (NH >= 0)
      {
          partialSpec = (fKs * NH) * fOs;
      }
      else
      {
        for (j = 0; j < 3; j++)
          partialSpec[j] = 0.0;
      }

      if (Environment::fPointLights[i].IsSpotLightEnabled() || Environment::fPointLights[i].AreFlapsEnabled())
      {
        //Don't evaluate if not in spot angle
        if (Environment::fPointLights[i].IsInSpot(lightDir) && Environment::fPointLights[i].IsInFlaps(inter))
          sum2 += Environment::fPointLights[i].GetAttenuation(inter) * Environment::fPointLights[i].GetIntensity(lightDir) * (partialDiff + partialSpec);
      }
      else
        sum2 += Environment::fPointLights[i].GetAttenuation(inter) * Environment::fPointLights[i].GetIntensity(lightDir) * (partialDiff + partialSpec);
    }

    //Now total with ambient
    fI = sum1 + sum2;

    //Normalize
    fI = NormalizeColor(fI);
  }
}
